@@ -19,7 +19,7 @@ import conda_build.config
 from .feedstock_outputs import request_copy, split_pkg
 
 
-def get_built_distribution_names(recipe_dir, variant):
+def get_built_distribution_names_and_subdirs(recipe_dir, variant):
     additional_config = {}
     for v in variant:
         variant_dir, base_name = os.path.split(v)
@@ -42,7 +42,8 @@ def get_built_distribution_names(recipe_dir, variant):
     for m in skipped_distributions:
         print("{} configuration was skipped in build/skip.".format(m.name()))
 
-    return set([m.name() for m, _, _ in metas if not m.skip()])
+    subdirs = set([m.config.target_subdir for m, _, _ in metas if not m.skip()])
+    return set([m.name() for m, _, _ in metas if not m.skip()]), subdirs
 
 
 @contextlib.contextmanager
@@ -86,12 +87,13 @@ def built_distribution_already_exists(cli, name, version, fname, owner):
     return exists
 
 
-def upload(token_fn, path, owner, channels):
-    subprocess.check_call(['anaconda', '--quiet', '-t', token_fn,
-                           'upload', path,
-                           '--user={}'.format(owner),
-                           '--channel={}'.format(channels)],
-                          env=os.environ)
+def upload(token_fn, path, owner, channels, private_upload=False):
+    cmd = ['anaconda', '--quiet', '-t', token_fn,
+           'upload', path, '--user={}'.format(owner),
+           '--channel={}'.format(channels)]
+    if private_upload:
+        cmd.append("--private")
+    subprocess.check_call(cmd,  env=os.environ)
 
 
 def delete_dist(token_fn, path, owner, channels):
@@ -131,7 +133,14 @@ def distribution_exists_on_channel(binstar_cli, meta, fname, owner, channel='mai
 
 
 def upload_or_check(
-    feedstock, recipe_dir, owner, channel, variant, validate=False, git_sha=None
+    feedstock,
+    recipe_dir,
+    owner,
+    channel,
+    variant,
+    validate=False,
+    git_sha=None,
+    private_upload=False,
 ):
     if validate and "STAGING_BINSTAR_TOKEN" in os.environ:
         token = os.environ["STAGING_BINSTAR_TOKEN"]
@@ -147,20 +156,16 @@ def upload_or_check(
 
     cli = get_server_api(token=token)
 
-    allowed_dist_names = get_built_distribution_names(recipe_dir, variant)
+    allowed_dist_names, allowed_subdirs = get_built_distribution_names_and_subdirs(recipe_dir, variant)
 
     # The list of built distributions
-    paths = (
-        [
-            os.path.join('noarch', p)
-            for p in os.listdir(os.path.join(conda_build.config.croot, 'noarch'))
-        ]
-        + [
-            os.path.join(conda_build.config.subdir, p)
-            for p in os.listdir(
-                os.path.join(conda_build.config.croot, conda_build.config.subdir))
-        ]
-    )
+    paths = set()
+    for subdir in list(allowed_subdirs) + [conda_build.config.subdir, 'noarch']:
+        if not os.path.exists(os.path.join(conda_build.config.croot, subdir)):
+            continue
+        for p in os.listdir(os.path.join(conda_build.config.croot, subdir)):
+            paths.add(os.path.join(subdir, p))
+
     built_distributions = [
         (
             split_pkg(path)[1],
@@ -215,7 +220,7 @@ def upload_or_check(
                     if not built_distribution_already_exists(
                         cli, name, version, path, owner
                     ):
-                        upload(token_fn, path, owner, channel)
+                        upload(token_fn, path, owner, channel, private_upload=private_upload)
                     else:
                         print(
                             'Distribution {} already exists for {}'.format(path, owner))
@@ -233,7 +238,14 @@ def upload_or_check(
 
 
 def retry_upload_or_check(
-    feedstock, recipe_dir, owner, channel, variant, validate=False, git_sha=None,
+    feedstock,
+    recipe_dir,
+    owner,
+    channel,
+    variant,
+    validate=False,
+    git_sha=None,
+    private_upload=False,
 ):
     # perform a backoff in case we fail.  THis should limit the failures from
     # issues with the Anaconda api
@@ -243,6 +255,7 @@ def retry_upload_or_check(
             res = upload_or_check(
                 feedstock, recipe_dir, owner, channel, variant,
                 validate=validate, git_sha=git_sha if i == n_try-1 else None,
+                private_upload=private_upload
             )
             return res
         except Exception as e:
